\\ Copyright 2022, 2023 Kevin Ryde

\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

default(sopath,".:/usr/lib:/usr/lib/i386-linux-gnu/");
primecount_verbose = 1;
read("primecount.gp");
read("primecount.gp");

option_verbose = 0
;
check_count = 0;
check_equal(got,want,name="") =
{
  check_count++;
  if(got != want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     option_verbose,
     print("ok "check_count" "name));
}

check_error(func,want,name="") =
{
  check_equal(iferr(my(ret=func());
                    Str("no error, return = ",ret),
                    e,
                    if(option_verbose,
                       print("error message: \"",e,"\""));
                    errname(e)),
              want,
              name);
}

test_is_program = (primecount_INTERNAL_program_run != 'primecount_INTERNAL_program_run);
print("test_is_program = ",test_is_program);
test_is_library = ! test_is_program;

\\-----------------------------------------------------------------------------
\\ primecount_version()

{
  print("primecount_verbose is: ", primecount_verbose);
  print("primecount_use_program is: ", primecount_use_program);

  print("primecount_version() is:");
  print(primecount_version());
}

\\-----------------------------------------------------------------------------
\\ primecount_INTERNAL_validate()

{
  if(test_is_program,
     for(i=0,9,
        primecount_INTERNAL_validate([Str(i)]));
     
     check_error(()->primecount_INTERNAL_validate(["1234x"]),
                 "e_USER",
                 "primecount_INTERNAL_validate() bad number");
    );
}

\\-----------------------------------------------------------------------------
\\ primecount limit
\\ eg. running  primecount -n 8446744073709551616

{
  my(limit=216289611853439384);
  check_equal(logint(limit,10), 17);
  check_equal(2*10^17 < limit, 1);
  check_equal(3*10^17 > limit, 1);
  check_equal(2^57 < limit, 1);
  check_equal(2^58 > limit, 1);
}


\\-----------------------------------------------------------------------------
\\ primecount_nth_prime()

{
  check_equal(vector(10,n,primecount_nth_prime(n)),
              primes(10));

  if(option_verbose,
     print("primecount_nth_prime() random"));
  for(rep=1,100,
     my(n=random(10^6));
     primecount_nth_prime(n) == prime(n) || error());

  if(test_is_library,
     check_error(()->primecount_nth_prime(0),
                 "e_DOMAIN",
                 "primecount_nth_prime() n=0 e_DOMAIN");
     \\ not sure the string is a guaranteed format
     if(0,
        check_equal(iferr(primecount_nth_prime(0),e,Str(e)),
                    "error(\"domain error in primecount_nth_prime: n < 1\")",
                    "primecount_nth_prime() n=0 message"));

     \\ x=2^63, bigger than int64_t
     check_error(()->primecount_nth_prime(2^63),
                 "e_OVERFLOW",
                 "primecount_nth_prime() x=2^63 e_OVERFLOW");

     \\ x=max+1 resulting prime too big for int64_t
     my(nth_prime_max_n = 216289611853439384);
     check_error(()->primecount_nth_prime(nth_prime_max_n + 1),
                 "e_OVERFLOW",
                 "primecount_nth_prime() max+1 e_OVERFLOW");

     check_error(()->primecount_nth_prime([]),
                 "e_TYPE",
                 "primecount_nth_prime() arg vector []");
     check_error(()->primecount_nth_prime('sym),
                 "e_TYPE",
                 "primecount_nth_prime() arg vector symbol");
     check_error(()->primecount_nth_prime(5.5),
                 "e_TYPE",
                 "primecount_nth_prime() arg flonum");
     );
}


\\-----------------------------------------------------------------------------
\\ primecount_pi()

{
  check_equal(primecount_pi(100), 25);
  check_equal(primecount_pi(100.5), 25);
  check_equal(primecount_pi(0), 0, "primecount_pi() 0");
  check_equal(primecount_pi(-1), 0);
  check_equal(primecount_pi(-100), 0);
  check_equal(primecount_pi(-2^1024), 0);

  if(test_is_library,

     \\ x too big
     check_equal(iferr(my(pi=primecount_pi(2^1024));
                       Str("no error, pi=",pi),
                       e,
                       if(option_verbose,
                          print("x too big message: \"",e,"\""));
                       errname(e)),
                 "e_OVERFLOW",
                 "primecount_pi() x too big want e_OVERFLOW");

     check_error(()->primecount_pi('sym),
                 "e_TYPE",
                 "primecount_pi() type symbol");
    );
}


\\-----------------------------------------------------------------------------
\\ primecount_phi()

{
  check_equal(primecount_phi(-100,-100), 0);
  check_equal(primecount_phi(100,-100), 0);
  check_equal(primecount_phi(-100,100), 0);
  check_equal(primecount_phi(0,100), 0);

  check_equal(primecount_phi(100,3),
              sum(i=1,100, gcd(i,2*3*5)==1),
              "primecount_phi() 100 3");

  my(divisible_by_any_of_vec(n,v) =
     for(i=1,#v, if(n%v[i]==0, return(1))); 0);
  my(by_count(x,a) =
     my(v=primes(a), ret=0);
     sum(i=1,x, ! divisible_by_any_of_vec(i,v)));
  check_equal(vector(20,x, vector(10,a, primecount_phi(x,a))),
              vector(20,x, vector(10,a, by_count(x,a))));

  if(test_is_library,
     check_error(()->primecount_phi(2^1024,3),
                 "e_OVERFLOW",
                 "primecount_phi() x too big e_OVERFLOW");
     check_error(()->primecount_phi(3,2^1024),
                 "e_OVERFLOW",
                 "primecount_phi() a too big e_OVERFLOW");

     check_error(()->primecount_phi('sym,123),
                 "e_TYPE",
                 "primecount_phi() type symbol x");
     check_error(()->primecount_phi(123,'sym),
                 "e_TYPE",
                 "primecount_phi() type symbol a");
    );
}


\\-----------------------------------------------------------------------------
check_equal(pi, 'pi, "no global variable");
check_equal(phi, 'phi, "no global variable");
check_equal(digit_set, 'digit_set, "digit_set not global variable");
print("end");
